#ifndef ELLIPTIC_ENERGY_HH
#define ELLIPTIC_ENERGY_HH

#include <dune/common/fmatrix.hh>
#include <dune/common/fvector.hh>
#include <dune/common/stdstreams.hh>

#include <dune/fufem/interval.hh>
#include <dune/fufem/arithmetic.hh>

#include "localfriction.hh"

template <size_t dim> class EllipticEnergy {
public:
  using LocalVector = Dune::FieldVector<double, dim>;
  using LocalMatrix = Dune::FieldMatrix<double, dim, dim>;

  using Nonlinearity = LocalFriction<dim>;

  EllipticEnergy(LocalMatrix const &A, LocalVector const &b,
                 std::shared_ptr<Nonlinearity const> phi,
                 typename Dune::BitSetVector<dim>::const_reference ignore)
      : A(A), b(b), phi(phi), ignore(ignore) {}

  double operator()(LocalVector const &v) const {
    return computeEnergy(A, v, b) + (*phi)(v);
  }

  LocalMatrix const &A;
  LocalVector const &b;
  std::shared_ptr<Nonlinearity const> const phi;
  /* Dimension that should be ignored; goes from 0 to dim-1; the
     special value dim means that no dimension should be ignored */
  typename Dune::BitSetVector<dim>::const_reference const ignore;

  void gradient(LocalVector const &x, LocalVector &y) const {
    A.mv(x, y);
    y -= b;
    phi->addGradient(x, y);

    for (size_t i = 0; i < dim; ++i)
      if (ignore[i])
        y[i] = 0;
  }
};
#endif
