#ifndef DUNE_TECTONIC_GLOBAL_RUINA_NONLINEARITY_HH
#define DUNE_TECTONIC_GLOBAL_RUINA_NONLINEARITY_HH

#include <vector>

#include <dune/common/bitsetvector.hh>
#include <dune/common/nullptr.hh>
#include <dune/common/fmatrix.hh>
#include <dune/common/fvector.hh>
#include <dune/istl/bcrsmatrix.hh>
#include <dune/istl/bvector.hh>

#include "globalnonlinearity.hh"
#include "frictionpotential.hh"

template <class Matrix, class Vector>
class GlobalRuinaNonlinearity : public GlobalNonlinearity<Matrix, Vector> {
public:
  using GlobalNonlinearity<Matrix, Vector>::block_size;
  using typename GlobalNonlinearity<Matrix, Vector>::Friction;

private:
  using typename GlobalNonlinearity<Matrix, Vector>::ScalarVector;

public:
  GlobalRuinaNonlinearity(Dune::BitSetVector<1> const &frictionalNodes,
                          ScalarVector const &nodalIntegrals,
                          FrictionData const &fd)
      : restrictions(nodalIntegrals.size()) {
    auto trivialNonlinearity =
        std::make_shared<Friction>(std::make_shared<TrivialFunction>());
    for (size_t i = 0; i < restrictions.size(); ++i) {
      restrictions[i] =
          frictionalNodes[i][0]
              ? std::make_shared<Friction>(
                    std::make_shared<FrictionPotential>(nodalIntegrals[i], fd))
              : trivialNonlinearity;
    }
  }

  void updateLogState(ScalarVector const &logState) override {
    for (size_t i = 0; i < restrictions.size(); ++i)
      restrictions[i]->updateLogState(logState[i]);
  }

  /*
    Return a restriction of the outer function to the i'th node.
  */
  std::shared_ptr<Friction> restriction(size_t i) const override {
    return restrictions[i];
  }

private:
  std::vector<std::shared_ptr<Friction>> restrictions;
};
#endif
