#ifndef DUNE_TECTONIC_TIMESTEPPING_NEWMARK_HH
#define DUNE_TECTONIC_TIMESTEPPING_NEWMARK_HH

template <class Vector, class Matrix, class Function, size_t dim>
class Newmark : public TimeSteppingScheme<Vector, Matrix, Function, dim> {
public:
  Newmark(Matrix const &_A, Matrix const &_M, Matrix const &_C,
          Vector const &_u_initial, Vector const &_v_initial,
          Vector const &_a_initial,
          Dune::BitSetVector<dim> const &_dirichletNodes,
          Function const &_dirichletFunction);

  void virtual nextTimeStep() override;
  void virtual setup(Vector const &, double, double, Vector &, Vector &,
                     Matrix &) override;
  void virtual postProcess(Vector const &) override;
  void virtual extractDisplacement(Vector &) const override;
  void virtual extractVelocity(Vector &) const override;

private:
  Matrix const &A;
  Matrix const &M;
  Matrix const &C;
  Vector u;
  Vector v;
  Vector a;
  Dune::BitSetVector<dim> const &dirichletNodes;
  Function const &dirichletFunction;

  Vector u_o;
  Vector v_o;
  Vector a_o;

  double tau;

  bool postProcessCalled = false;
};
#endif
